package org.communiquons.dccaggregator.activities

import android.os.Bundle
import android.util.Log
import android.view.MenuItem
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanOptions
import org.communiquons.dccaggregator.R
import org.communiquons.dccaggregator.WALLET_URL
import org.communiquons.dccaggregator.databinding.ActivityCertsManagerBinding
import org.communiquons.dccaggregator.models.Certificate
import java.net.URLDecoder
import java.nio.charset.StandardCharsets

class CertsManager : AppCompatActivity() {

    private lateinit var binding: ActivityCertsManagerBinding

    private val TAG = CertsManager::class.java.canonicalName

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityCertsManagerBinding.inflate(layoutInflater)
        setContentView(binding.root)

        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        binding.scanCertButton.setOnClickListener {
            addNewCertificate()
        }


        intent?.dataString.toString().apply {
            if (this.startsWith(WALLET_URL))
                processGivenCertificate(this)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            finish()
            return true
        }

        return super.onOptionsItemSelected(item)
    }

    private val barcodeLauncher = registerForActivityResult(ScanContract()) {
        if (it.contents == null)
            return@registerForActivityResult

        processGivenCertificate(it.contents)
    }

    private fun addNewCertificate() {
        barcodeLauncher.launch(ScanOptions().apply {
            setOrientationLocked(true)
            setDesiredBarcodeFormats(ScanOptions.QR_CODE)
            setPrompt(resources.getString(R.string.scan_qr_prompt))
        })
    }

    private fun processGivenCertificate(input: String) {
        // Decode input if required
        val decoded = if (input.startsWith(WALLET_URL)) {
            URLDecoder.decode(input.replace(WALLET_URL, ""), StandardCharsets.UTF_8.displayName())
        } else {
            input
        }

        try {
            val cert = Certificate(decoded)

            Log.v(TAG, cert.fullName)

        } catch (e: Exception) {
            e.printStackTrace()
            Toast.makeText(this, R.string.invalid_certificate, Toast.LENGTH_SHORT).show()
            return
        }
    }
}